@@ -17,7 +17,7 @@ from skimage import draw
 import mxnet as mx
 from mxnet import nd
 
-from . import utils, transforms, models, guiparts, plot, menus, io
+from . import utils, transforms, models, guiparts, plot, menus, io, dynamics
 
 try:
     from google.cloud import storage
@@ -407,16 +407,58 @@ class MainW(QtGui.QMainWindow):
         self.progress.setStyleSheet('color: gray;')
         self.l0.addWidget(self.progress, b,0,1,2)
 
+        # post-hoc paramater tuning
+
+        b+=1
+        label = QtGui.QLabel('flow error threshold:')
+        label.setToolTip('threshold on flow error to accept for masks (set higher to get more cells)')
+        label.setStyleSheet('color: white;')
+        self.l0.addWidget(label, b, 0,1,2)
+
+        b+=1
+        self.threshold = 0.4
+        self.threshslider = QtGui.QSlider()
+        self.threshslider.setOrientation(QtCore.Qt.Horizontal)
+        self.threshslider.setMinimum(1.0)
+        self.threshslider.setMaximum(30.0)
+        self.threshslider.setValue(4)
+        self.l0.addWidget(self.threshslider, b, 0,1,2)
+        self.threshslider.valueChanged.connect(self.compute_cprob)
+        #self.threshslider.setEnabled(False)
+        
+        b+=1
+        label = QtGui.QLabel('cell prob threshold:')
+        label.setStyleSheet('color: white;')
+        self.l0.addWidget(label, b, 0,1,2)
+        label.setToolTip('cell probability threshold (set lower to get more cells)')
+        
+        b+=1
+        self.probslider = QtGui.QSlider()
+        self.probslider.setOrientation(QtCore.Qt.Horizontal)
+        self.probslider.setMinimum(-6.0)
+        self.probslider.setMaximum(6.0)
+        self.probslider.setValue(0.0)
+        self.cellprob = 0.0
+        self.l0.addWidget(self.probslider, b, 0,1,2)
+        self.probslider.valueChanged.connect(self.compute_cprob)
+        #self.probslider.setEnabled(False)
+
+
+        b+=1
+        line = QHLine()
+        line.setStyleSheet('color: white;')
+        self.l0.addWidget(line, b,0,1,2)
+
         self.autobtn = QtGui.QCheckBox('auto-adjust')
         self.autobtn.setStyleSheet(self.checkstyle)
         self.autobtn.setChecked(True)
         self.l0.addWidget(self.autobtn, b+2,0,1,1)
 
         b+=1
-        label = QtGui.QLabel('saturation')
+        label = QtGui.QLabel('Image saturation:')
         label.setStyleSheet(self.headings)
         label.setFont(self.boldfont)
-        self.l0.addWidget(label, b,1,1,1)
+        self.l0.addWidget(label, b,0,1,2)
 
         b+=1
         self.slider = guiparts.RangeSlider(self)
@@ -1064,6 +1106,40 @@ class MainW(QtGui.QMainWindow):
             print(self.current_model)
             self.model = models.Cellpose(device=device, model_type=self.current_model)
 
+    def compute_cprob(self):
+        rerun = False
+        if self.cellprob != self.probslider.value():
+            rerun = True
+            self.cellprob = self.probslider.value()
+        if self.threshold != self.threshslider.value()/10.:
+            rerun = True
+            self.threshold = self.threshslider.value()/10.
+        if not rerun:
+            return
+        
+        if self.threshold==3.0 or self.NZ>1:
+            thresh = None
+            print('computing masks with cell prob=%0.3f, no flow error threshold'%
+                    (self.cellprob))
+        else:
+            thresh = self.threshold
+            print('computing masks with cell prob=%0.3f, flow error threshold=%0.3f'%
+                    (self.cellprob, thresh))
+        maski = dynamics.get_masks(self.flows[3].copy(), iscell=(self.flows[4][-1]>self.cellprob),
+                                    flows=self.flows[4][:-1], threshold=thresh)
+        if self.NZ==1:
+            maski = dynamics.fill_holes(maski)
+
+        self.masksOn = True
+        self.outlinesOn = True
+        self.MCheckBox.setChecked(True)
+        self.OCheckBox.setChecked(True)
+        if maski.ndim<3:
+            maski = maski[np.newaxis,...]
+        print('%d cells found'%(len(np.unique(maski)[1:])))
+        io._masks_to_gui(self, maski, outlines=None)
+        self.show()
+
     def compute_model(self):
         self.progress.setValue(0)
         if 1:
@@ -1078,7 +1154,7 @@ class MainW(QtGui.QMainWindow):
                 do_3D = True
                 data = self.stack.copy()
             else:
-                data = [self.stack[0].copy()]
+                data = self.stack[0].copy()
             channels = self.get_channels()
             self.diameter = float(self.Diameter.text())
             try:
@@ -1092,9 +1168,11 @@ class MainW(QtGui.QMainWindow):
 
             self.progress.setValue(75)
 
+            #if not do_3D:
+            #    masks = masks[0][np.newaxis,:,:]
+            #    flows = flows[0]
             if not do_3D:
-                masks = masks[0][np.newaxis,:,:]
-                flows = flows[0]
+                masks = masks[np.newaxis,...]
             self.flows[0] = flows[0]
             self.flows[1] = (np.clip(utils.normalize99(flows[2]),0,1) * 255).astype(np.uint8)
             if not do_3D:
@@ -1102,9 +1180,12 @@ class MainW(QtGui.QMainWindow):
                 self.flows = [self.flows[n][np.newaxis,...] for n in range(len(self.flows))]
             else:
                 self.flows[2] = (flows[1][0]/10 * 127 + 127).astype(np.uint8)
+            if len(flows)>2:
+                self.flows.append(flows[3])
+                self.flows.append(np.concatenate((flows[1], flows[2][np.newaxis,...]), axis=0))
+                print(self.flows[3].shape, self.flows[4].shape)
 
-
-            print('%d cells found with unet'%(len(np.unique(masks)[1:])))
+            print('%d cells found with cellpose net'%(len(np.unique(masks)[1:])))
             self.progress.setValue(80)
             z=0
             self.masksOn = True
